{	This program is copyright (c) 1993 - 2005 Jos Kunst, the Jos Kunst heirs.	}
{	This program is free software. You can redistribute it and/or modify it 	}
{	under the terms of version 2 of the GNU General Public License, which you	}
{	should have received with it.												}
{	This program is distributed in the hope that it will be useful, but			}
{	without any warranty, expressed or implied.									}

unit outputs;

interface
	uses
		translations, chordscanner, chordworker;

	const
		m = 24;
		n = 8;
		ensembletop = 103;{108;96;}
		ensemblebottom = 36;{21;34;}

	type
{instruments = (fl, ob, cl, fag, cor, trp, trbn, sax1, sax2, sax3, sax4, vla1, vla2);}
		instruments = (vl1, vl2, vla, vlc);
		instrset = set of instruments;
{    fingerparts = (LH5, LH4, LH3, LH2, LH1, RH1, RH2, RH3, RH4, RH5);}
{    handsset = set of fingerparts;}
		position = array[1..m, 1..n] of byte;
		finalarr = file of position;
		chintarr = array[1..16, 1..12] of byte;
		memfile = file of chintarr;
		scorebook = array[1..64] of position;
		scorefile = file of scorebook;
{if scorebook >= 128 then "variables of this type" (i.e. the scorefile type) "would be too large"}

	var
		storenewsit, printfile, newproduction, showmem, simplify: boolean;
		orchords: text;
		proposal, melfix, adsg, play: byte;
		pitchatom, duratom, dynatom: byte;
		transtarget, arttarget, dyntarget, transprecision, artprecision, dynprecision: byte;
		maxcardvl1, maxcardvl2, maxcardvla, maxcardvlc: byte;
		banonunisono, lotohi, maxcont, preferthirds, thirdtimearound, nooverlap: boolean;
		notr32nds, no32nds, notr16ths, no16ths, notr8ths, no8ths: boolean;
		orchtot: instrset;
{    hands: handsset;}
		strqt: instrset;
		basedur: real;

	procedure decidewhethernewfinalpos;
{in order to permit dry runs, asks the user whether he wants the old finalposfiles to be overwritten}
	procedure decidewhethertoprint;
{in order to permit dry runs, asks the user whether he wants the chordfile to be overwritten}
	procedure initialize;
{the text window; the randseed; the three preceding procedures; and the instruments involved}
	function name (instrument: instruments): string;
{replaces the scalar type representing the instruments by strings containing their names}
	procedure showpos (p: position);
{writes orchestrated chord progression to screen, and perhaps to printer as well}
	procedure writefinalstodisk (p: position;
									me: chintarr);
{writes up to 10 resulting new finalposfiles, with corresponding memfiles keeping track of chord repeats}
	procedure filloutpos (var p: position);
{given a position file with only orchestrated pitches, completes it with corresponding chordscanner results}
	procedure subinitialize (var lastpos: position;
									var linkedchord: pitchset;
									var memspan: chintarr);
{re-reads the starting position in order to come up with a new proposal}
	procedure writechordfile;
{closes the chordfile containing all proposals}

implementation

	procedure decidewhethernewfinalpos;
		var
			ch: char;
	begin
		write('Do you want new final position written to disk? (n/y): -->');
		readln(ch);
		if (ch = 'Y') or (ch = 'y') then
			storenewsit := TRUE
		else
			storenewsit := FALSE
	end;

	procedure decidewhethertoprint;
		var
			ch: char;
	begin
		write('Do you want results stored in printable diskfile? (n/y): -->');
		readln(ch);
		if (ch = 'Y') or (ch = 'y') then
			printfile := TRUE
		else
			printfile := FALSE;
		if printfile then
			begin
				open(orchords, 'chordfile');
				rewrite(orchords)
			end
	end;

	procedure initialize;
		var
			fullScreen: rect;
	begin
		SetRect(fullScreen, 9, 44, 502, 856);
		SetTextRect(fullScreen);
		ShowText;
		showmem := FALSE; {default}
		randseed := integer(tickcount);
		storenewsit := FALSE; {default}
{--------if you want to decide on the fly---------}
		decidewhethernewfinalpos;
		printfile := FALSE; {default}
{--------if you want to decide on the fly---------}
		decidewhethertoprint;
{orchtot := [fl, ob, cl, fag, sax1, sax2, sax3, sax4, cor, trp, trbn, vla1, vla2];}
{    hands := [LH5, LH4, LH3, LH2, LH1, RH1, RH2, RH3, RH4, RH5];}
		strqt := [vl1, vl2, vla, vlc];
	end;

	function name (instrument: instruments): string;
	begin
		if instrument = vl1 then
			name := 'vl1'
		else if instrument = vl2 then
			name := 'vl2'
		else if instrument = vla then
			name := 'vla'
		else if instrument = vlc then
			name := 'vlc'
	end;

	procedure showpos (p: position);
		var
			i: instruments;
			s: byte;
	begin{showpos}
{-------line one: one digit for the beat number in the measure: 1, 2, 3, 4, 1, ...:  -----------------}
		write(p[m - 1, 2] : 1);
{--------------two digits for the position within the beat: range 0..15: -----------------------}
		if p[m, 2] < 10 then
			write('0', p[m, 2] : 1)
		else
			write(p[m, 2] : 2);
{-------------... followed by instruments & pitches: ----------------}
		for i := vl1 to vlc do
			for s := 1 to 4 do
				if p[ord(i) * 5 + s, 1] > 1 then
					begin
						if s = 1 then
							write(' |', name(i), ':');
						write(nameof(p[ord(i) * 5 + s, 1]) : 3)
					end;
		writeln(' //DSG:', p[m, 4] : 2, ' HR:', nameof(p[m, 1]) : 3);
{---------------now for line 2; 3 digits for the measure number: -------------------------}
		if p[m - 2, 2] < 10 then
			write('00', p[m - 2, 2] : 1)
		else if p[m - 2, 2] < 100 then
			write('0', p[m - 2, 2] : 2)
		else
			write(p[m - 2, 2] : 3);
		for i := vl1 to vlc do
			if p[ord(i) * 5 + 1, 1] > 1 then
				write(' |', name(i), ':', p[ord(i) * 5 + 1, 5] : 2, p[ord(i) * 5 + 1, 6] : 2, p[ord(i) * 5 + 1, 7] : 2);
		writeln;
		if printfile then{------- write these same things to the disk -------------}
			begin
				write(orchords, p[m - 1, 2] : 1);
				if p[m, 2] < 10 then
					write(orchords, '0', p[m, 2] : 1)
				else
					write(orchords, p[m, 2] : 2);
				for i := vl1 to vlc do
					for s := 1 to 4 do
						if p[ord(i) * 5 + s, 1] > 1 then
							begin
								if s = 1 then
									write(orchords, ' |', name(i), ': ');
								write(orchords, nameof(p[ord(i) * 5 + s, 1]) : 3)
							end;
				writeln(orchords, ' //DSG:', p[m, 4] : 2, ' HR:', nameof(p[m, 1]) : 3);
				if p[m - 2, 2] < 10 then
					write(orchords, '00', p[m - 2, 2] : 1)
				else if p[m - 2, 2] < 100 then
					write(orchords, '0', p[m - 2, 2] : 2)
				else
					write(orchords, p[m - 2, 2] : 3);
				for i := vl1 to vlc do
					if p[ord(i) * 5 + 1, 1] > 1 then
						write(orchords, ' |', name(i), ': ', p[ord(i) * 5 + 1, 5] : 2, p[ord(i) * 5 + 1, 6] : 2, p[ord(i) * 5 + 1, 7] : 2);
				writeln(orchords)
			end
	end;

	procedure writefinalstodisk (p: position;
									me: chintarr);
		var
			newfinalpos: finalarr;
			memoryfile: memfile;
	begin
		if storenewsit then
			begin
				case proposal of
					1: 
						begin
							open(newfinalpos, 'newfinalposfile1');
							open(memoryfile, 'newmemfile1')
						end;
					2: 
						begin
							open(newfinalpos, 'newfinalposfile2');
							open(memoryfile, 'newmemfile2')
						end;
					3: 
						begin
							open(newfinalpos, 'newfinalposfile3');
							open(memoryfile, 'newmemfile3')
						end;
					4: 
						begin
							open(newfinalpos, 'newfinalposfile4');
							open(memoryfile, 'newmemfile4')
						end;
					5: 
						begin
							open(newfinalpos, 'newfinalposfile5');
							open(memoryfile, 'newmemfile5')
						end;
					6: 
						begin
							open(newfinalpos, 'newfinalposfile6');
							open(memoryfile, 'newmemfile6')
						end;
					7: 
						begin
							open(newfinalpos, 'newfinalposfile7');
							open(memoryfile, 'newmemfile7')
						end;
					8: 
						begin
							open(newfinalpos, 'newfinalposfile8');
							open(memoryfile, 'newmemfile8')
						end;
					9: 
						begin
							open(newfinalpos, 'newfinalposfile9');
							open(memoryfile, 'newmemfile9')
						end;
					10: 
						begin
							open(newfinalpos, 'newfinalposfile10');
							open(memoryfile, 'newmemfile10')
						end
				end;
				rewrite(newfinalpos);
				write(newfinalpos, p);
				close(newfinalpos);
				rewrite(memoryfile);
				write(memoryfile, me);
				close(memoryfile)
			end
	end;

	procedure filloutpos (var p: position);
		var
			cs: pitchset;
			cr: chordrec;
			c, cc: byte;
	begin
		cs := [];
		for c := 1 to m - 4 do
			if c mod 5 <> 0 then
				if p[c, 1] > 0 then
					cs := cs + [p[c, 1]];
		scan(cs, cr);
		carr := cr.stru;
		dsg := cr.diss;
		totalofall := cr.total;
		hroot := cr.hifu;
		p[m, 4] := dsg;  { dsg invullen }
		p[m, 1] := hroot;   { Hindemith grondtoon invullen }
		c := 1;
		repeat   { dsgbijdragen invullen }
			for cc := 1 to m - 4 do
				if p[cc, 1] = carr[c, 1] then
					p[cc, 4] := carr[c, 2];
			c := c + 1;
		until carr[c, 1] = 0;
	end;

	procedure subinitialize (var lastpos: position;
									var linkedchord: pitchset;
									var memspan: chintarr);
		var
			count, cont: byte;
			oldfinalpos: finalarr;
			oldmemory: memfile;
	begin
		open(oldfinalpos, 'oldfinalposfile');
		reset(oldfinalpos);
		read(oldfinalpos, lastpos);
		close(oldfinalpos);
{if lastpos[m, 4] = 0 then}
		filloutpos(lastpos);
		open(oldmemory, 'oldmemfile');
		reset(oldmemory);
{    read(oldmemory, memspan); zzzzzz dit moet straks wel}
		close(oldmemory);
		if newproduction then
			for count := 1 to 16 do
				for cont := 1 to 12 do
					memspan[count, cont] := 0;
		linkedchord := [];
		for count := 1 to m - 4 do
			if count mod 5 <> 0 then
				if lastpos[count, 1] > 0 then
					linkedchord := linkedchord + [lastpos[count, 1]];
		writeln('PROPOSAL ', proposal : 2);
		if printfile then
			writeln(orchords, 'PROPOSAL ', proposal : 2);
		showpos(lastpos);
		if lastpos[m, 8] = 1 then
			banonunisono := TRUE
		else
			banonunisono := FALSE;
		if lastpos[m - 1, 8] = 1 then
			lotohi := TRUE
		else
			lotohi := FALSE;
		if lastpos[m - 2, 8] = 1 then
			maxcont := TRUE
		else
			maxcont := FALSE;
		if lastpos[m - 3, 8] = 1 then
			preferthirds := TRUE
		else
			preferthirds := FALSE;
		if lastpos[m - 4, 8] = 1 then
			simplify := TRUE
		else
			simplify := FALSE;
		if lastpos[m - 5, 8] = 1 then
			nooverlap := TRUE
		else
			nooverlap := FALSE;
		maxcardvl1 := lastpos[1, 8];
		maxcardvl2 := lastpos[6, 8];
		maxcardvla := lastpos[11, 8];
		maxcardvlc := lastpos[16, 8];
		if lastpos[m - 3, 2] = 1 then
			notr32nds := TRUE
		else
			notr32nds := FALSE;
		if lastpos[m - 3, 3] = 1 then
			no32nds := TRUE
		else
			no32nds := FALSE;
		if lastpos[m - 3, 4] = 1 then
			notr16ths := TRUE
		else
			notr16ths := FALSE;
		if lastpos[m - 3, 5] = 1 then
			no16ths := TRUE
		else
			no16ths := FALSE;
		if lastpos[m - 3, 6] = 1 then
			notr8ths := TRUE
		else
			notr8ths := FALSE;
		if lastpos[m - 3, 7] = 1 then
			no8ths := TRUE
		else
			no8ths := FALSE;
{------------miscellaneous constants -----------}
		melfix := lastpos[m - 1, 1];
		hroot := lastpos[m, 1];
		basedur := lastpos[m - 1, 4];
		duratom := lastpos[m - 1, 3];
		adsg := lastpos[m - 2, 4];
		pitchatom := lastpos[m - 2, 3];
		dsg := lastpos[m, 4];
		play := lastpos[m - 2, 1];
		transtarget := lastpos[m - 1, 5];
		transprecision := lastpos[m - 2, 5];
		arttarget := lastpos[m - 1, 6];
		artprecision := lastpos[m - 2, 6];
		dyntarget := lastpos[m - 1, 7];
		dynprecision := lastpos[m - 2, 7];
		dynatom := lastpos[m, 3];
	end;

	procedure writechordfile;
	begin
		close(orchords)
	end;

end.